
local directory = g_currentModDirectory
local modName = g_currentModName
extendedSelling = {}
extendedSelling.NotificationCount = 0
extendedSelling.lastResetHour = -1
--extendedSelling.candidates = {}
--extendedSelling.activeObjects = {}

function extendedSelling.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations)
end
function extendedSelling.registerEventListeners(vehicleType)
	for _, functionName in pairs( { "onLoad", "onLoadFinished" } ) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, extendedSelling)
	end
end
function extendedSelling.registerFunctions(vehicleType)
	--SpecializationUtil.registerFunction(vehicleType, "ess_minuteChanged", extendedSelling.ess_minuteChanged)
	SpecializationUtil.registerFunction(vehicleType, "ess_hourChanged", extendedSelling.ess_hourChanged)
	SpecializationUtil.registerFunction(vehicleType, "isActiveObject", extendedSelling.isActiveObject)
end
function extendedSelling.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getSellPrice", extendedSelling.getSellPrice)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getSellPrice_RVBClone", extendedSelling.getSellPrice_RVBClone)
end
function extendedSelling:onLoad(savegame)
	self.initialized = false
	self.ownedItemsProfitLoss = { plus_minus = 0, percentage = 0 }
	--g_messageCenter:subscribe(MessageType.MINUTE_CHANGED, extendedSelling.ess_minuteChanged, self)
	g_messageCenter:subscribe(MessageType.HOUR_CHANGED, extendedSelling.ess_hourChanged, self)
	if g_modIsLoaded["FS25_gameplay_Real_Vehicle_Breakdowns"] then
		local delayedStart
		delayedStart = {
			startUpdateTime = 2000,
			update = function(self, dt)
				self.startUpdateTime = self.startUpdateTime - dt
				if self.startUpdateTime < 0 and not g_gui:getIsGuiVisible() then
					FS25_gameplay_Real_Vehicle_Breakdowns.rvbWorkshopScreen.setVehicleDetails = Utils.appendedFunction(FS25_gameplay_Real_Vehicle_Breakdowns.rvbWorkshopScreen.setVehicleDetails, extendedSelling.rvbWorkshopScreen_setVehicleDetails)
					removeModEventListener(self)
					delayedStart = nil
				end
			end
		}
		addModEventListener(delayedStart)
	end
end
function extendedSelling:onLoadFinished(savegame)
	self.ownedItemsProfitLoss.plus_minus = math.random(0, 99)
	self.ownedItemsProfitLoss.percentage = math.random(2, 25)
end
function extendedSelling:isActiveObject(vehicle)
	for _, v in ipairs(extendedSelling.activeObjects) do
		if v == vehicle then
			return true
		end
	end
	return false
end
function extendedSelling.selectRandomExtendedSellingVehicles()
	extendedSelling.candidates = {}
	extendedSelling.activeObjects = {}
	local vehicles = g_currentMission.vehicleSystem.vehicles
	for _, vehicle in ipairs(vehicles) do
		local key = "spec_" .. modName .. ".extendedSelling"
		if vehicle[key] ~= nil then
			table.insert(extendedSelling.candidates, vehicle)
		end
	end
	for i = #extendedSelling.candidates, 2, -1 do
		local j = math.random(1, i)
		extendedSelling.candidates[i], extendedSelling.candidates[j] = extendedSelling.candidates[j], extendedSelling.candidates[i]
	end
	for i = 1, math.min(5, #extendedSelling.candidates) do
		table.insert(extendedSelling.activeObjects, extendedSelling.candidates[i])
	end
end
--function extendedSelling:ess_minuteChanged()
	--self.ownedItemsProfitLoss.plus_minus = math.random(0, 99)
	--self.ownedItemsProfitLoss.percentage = math.random(2, 25)
	--g_workshopScreen.list:reloadData()
	--g_inGameMenu.pageStatistics:updateVehicles()
--end
function extendedSelling:ess_hourChanged()
	local currentHour = g_currentMission.environment.currentHour
	if currentHour ~= extendedSelling.lastResetHour then
		extendedSelling.NotificationCount = 0
		extendedSelling.lastResetHour = currentHour
		extendedSelling.selectRandomExtendedSellingVehicles()
	end
	if extendedSelling:isActiveObject(self) and self.ownedItemsProfitLoss.plus_minus ~= 0 and self.ownedItemsProfitLoss.percentage ~= 0 
	and g_currentMission.environment.currentHour < 19 and g_currentMission.environment.currentHour > 7 then
		self.ownedItemsProfitLoss.plus_minus = math.random(0, 99)
		self.ownedItemsProfitLoss.percentage = math.random(20, 25)
		g_workshopScreen.list:reloadData()
		g_inGameMenu.pageStatistics:updateVehicles()
		if (self.ownedItemsProfitLoss.plus_minus % 2 == 0) and self.ownedItemsProfitLoss.percentage > 20 and self.propertyState == VehiclePropertyState.OWNED
		and (extendedSelling.NotificationCount or 0) < 3 then
			if g_currentMission.hud ~= nil then
				local message = string.format(g_i18n:getText("demandIncrease"), self:getFullName(), self.ownedItemsProfitLoss.percentage)
				g_currentMission.hud:addSideNotification(FSBaseMission.INGAME_NOTIFICATION_OK, message, 10000, GuiSoundPlayer.SOUND_SAMPLES.SUCCESS)
			end
			extendedSelling.NotificationCount = (extendedSelling.NotificationCount or 0) + 1
		end
	end
end
function extendedSelling:getSellPrice(superFunc)
	local sellPrice = superFunc(self)
	if extendedSelling:isActiveObject(self) and self.ownedItemsProfitLoss.plus_minus ~= 0 and self.ownedItemsProfitLoss.percentage ~= 0 
	and g_currentMission.environment.currentHour < 19 and g_currentMission.environment.currentHour > 7 then
		local ess_amount = (sellPrice / 100) * self.ownedItemsProfitLoss.percentage
		local PL_price = 0
		if self.ownedItemsProfitLoss.plus_minus <= 33 then
			PL_price = sellPrice
		elseif self.ownedItemsProfitLoss.plus_minus <= 66 then
			if (self.ownedItemsProfitLoss.plus_minus % 2 == 0) then
				PL_price = sellPrice + ess_amount
			else
				PL_price = sellPrice - ess_amount
			end
		else
			if (self.ownedItemsProfitLoss.plus_minus % 2 == 0) then
				PL_price = sellPrice + ess_amount
			else
				PL_price = sellPrice - ess_amount
			end
		end
		return PL_price
	else
		return sellPrice
	end
end
function extendedSelling:getSellPrice_RVBClone(superFunc)
	if g_modIsLoaded["FS25_gameplay_Real_Vehicle_Breakdowns"] then
		local storeItem = g_storeManager:getItemByXMLFilename(self.configFileName)
		local sellPrice = FS25_gameplay_Real_Vehicle_Breakdowns.VehicleBreakdowns.calculateSellPriceClone(storeItem, self.age, self.operatingTime, self:getPrice(), self:getRepairPrice(), self:getRepairPrice_RVBClone(), self:getRepaintPrice())
		if extendedSelling:isActiveObject(self) and self.ownedItemsProfitLoss.plus_minus ~= 0 and self.ownedItemsProfitLoss.percentage ~= 0 
		and g_currentMission.environment.currentHour < 19 and g_currentMission.environment.currentHour > 7 then
			local ess_amount = (sellPrice / 100) * self.ownedItemsProfitLoss.percentage
			local PL_price = 0
			if self.ownedItemsProfitLoss.plus_minus <= 33 then
				PL_price = sellPrice
			elseif self.ownedItemsProfitLoss.plus_minus <= 66 then
				if (self.ownedItemsProfitLoss.plus_minus % 2 == 0) then
					PL_price = sellPrice + ess_amount
				else
					PL_price = sellPrice - ess_amount
				end
			else
				if (self.ownedItemsProfitLoss.plus_minus % 2 == 0) then
					PL_price = sellPrice + ess_amount
				else
					PL_price = sellPrice - ess_amount
				end
			end
			return PL_price
		else
			return sellPrice
		end
	end
end
function extendedSelling.WorkshopScreen_setVehicleDetails(p76, p77, p78)
	local v79, v80
	if p77 == nil then
		v79 = 0
		v80 = 0
	else
		v79 = p77:getOperatingTime()
		v80 = p77.age
		if p77.propertyState == VehiclePropertyState.OWNED then
			local v81 = p77:getSellPrice() * EconomyManager.DIRECT_SELL_MULTIPLIER
			local v82 = math.floor(v81)
			local v83 = math.min(v82, p77:getPrice())
			--p78:getAttribute("priceText"):setText(g_i18n:formatMoney(v83))
			if extendedSelling:isActiveObject(p77) and p77.ownedItemsProfitLoss.plus_minus ~= 0 and p77.ownedItemsProfitLoss.percentage ~= 0 
			and g_currentMission.environment.currentHour < 19 and g_currentMission.environment.currentHour > 7 then
				local priceStr = "-"
				if (p77.ownedItemsProfitLoss.plus_minus % 2 == 0) then
					priceStr = string.format("%s (+%d%%)", g_i18n:formatMoney(v81), p77.ownedItemsProfitLoss.percentage)
				else
					priceStr = string.format("%s (-%d%%)", g_i18n:formatMoney(v81), p77.ownedItemsProfitLoss.percentage)
				end
				p78:getAttribute("priceText"):setText(priceStr)
			else
				p78:getAttribute("priceText"):setText(g_i18n:formatMoney(v83))
			end
		end
	end
end
WorkshopScreen.setVehicleDetails = Utils.appendedFunction(WorkshopScreen.setVehicleDetails, extendedSelling.WorkshopScreen_setVehicleDetails)
function extendedSelling.rvbWorkshopScreen_setVehicleDetails(p74, p75, p76)
	local v77, v78
	local rvb = p75.spec_faultData
	if p75 == nil then
		v77 = 0
		v78 = 0
	else
		v77 = p75:getOperatingTime()
		v78 = p75.age
		if p75.propertyState == VehiclePropertyState.OWNED then
			local v79 = p75:getSellPrice() * EconomyManager.DIRECT_SELL_MULTIPLIER
			local v80 = math.floor(v79)
			local v81 = math.min(v80, p75:getPrice())
			--p76:getAttribute("priceText"):setText(g_i18n:formatMoney(v81))
			if extendedSelling:isActiveObject(p75) and p75.ownedItemsProfitLoss.plus_minus ~= 0 and p75.ownedItemsProfitLoss.percentage ~= 0 
			and g_currentMission.environment.currentHour < 19 and g_currentMission.environment.currentHour > 7 then
				local priceStr = "-"
				if (p75.ownedItemsProfitLoss.plus_minus % 2 == 0) then
					priceStr = string.format("%s (+%d%%)", g_i18n:formatMoney(v81), p75.ownedItemsProfitLoss.percentage)
				else
					priceStr = string.format("%s (-%d%%)", g_i18n:formatMoney(v81), p75.ownedItemsProfitLoss.percentage)
				end
				p76:getAttribute("priceText"):setText(priceStr)
			else
				p76:getAttribute("priceText"):setText(g_i18n:formatMoney(v81))
			end	
		end
	end
end
function extendedSelling.InGameMenuStatisticsFrame_updateVehicles(p78)
	p78.vehicles = {}
	if g_localPlayer ~= nil then
		local v79 = g_currentMission
		local v80 = v79.accessHandler
		for _, v81 in ipairs(v79.vehicleSystem.vehicles) do
			if v80:canPlayerAccess(v81) and (v81:getShowInVehiclesOverview() and g_localPlayer.farmId == v81:getOwnerFarmId()) then
				local v82 = {
					["vehicle"] = v81,
					["columns"] = {}
				}
				local v83 = v81:getFullName()
				v82.columns[InGameMenuStatisticsFrame.COLUMN_NAME] = {
					["text"] = v83,
					["value"] = v83
				}
				local v84 = {
					["text"] = Vehicle.getSpecValueAge(nil, v81),
					["value"] = v81.age
				}
				v82.columns[InGameMenuStatisticsFrame.COLUMN_AGE] = v84
				local v85, v86
				if v81.getOperatingTime == nil then
					v85 = "-"
					v86 = 0
				else
					v85 = Vehicle.getSpecValueOperatingTime(nil, v81)
					v86 = v81:getOperatingTime()
				end
				v82.columns[InGameMenuStatisticsFrame.COLUMN_HOURS] = {
					["text"] = v85,
					["value"] = v86
				}
				local v87, v88
				if SpecializationUtil.hasSpecialization(Wearable, v81.specializations) then
					v87 = v81:getDamageAmount()
					local v89 = g_i18n
					local v90 = (1 - v87) * 100
					v88 = v89:formatNumber(math.ceil(v90), 0) .. " %"
				else
					v88 = "-"
					v87 = 0
				end
				v82.columns[InGameMenuStatisticsFrame.COLUMN_DAMAGE] = {
					["text"] = v88,
					["value"] = v87
				}
				local v91, v92
				if v81.propertyState == VehiclePropertyState.LEASED then
					v91 = v81.price * (EconomyManager.DEFAULT_RUNNING_LEASING_FACTOR + EconomyManager.PER_DAY_LEASING_FACTOR)
					v92 = g_i18n:formatMoney(v91)
				else
					v92 = "-"
					v91 = 0
				end
				v82.columns[InGameMenuStatisticsFrame.COLUMN_LEASING] = {
					["text"] = v92,
					["value"] = v91
				}
				local v93, v94
				if v81.propertyState == VehiclePropertyState.OWNED then
					--v93 = v81:getSellPrice()
					--v94 = g_i18n:formatMoney(v93)
					if extendedSelling:isActiveObject(v81) and v81.ownedItemsProfitLoss.plus_minus ~= 0 and v81.ownedItemsProfitLoss.percentage ~= 0 
					and g_currentMission.environment.currentHour < 19 and g_currentMission.environment.currentHour > 7 then
						local rvb = v81.spec_faultData
						if rvb ~= nil then
							v93 = v81:getSellPrice_RVBClone()
						else
							v93 = v81:getSellPrice()
						end
						if (v81.ownedItemsProfitLoss.plus_minus % 2 == 0) then
							v94 = string.format("%s (+%d%%)", g_i18n:formatMoney(v93, 0, true, true), v81.ownedItemsProfitLoss.percentage)
						else
							v94 = string.format("%s (-%d%%)", g_i18n:formatMoney(v93, 0, true, true), v81.ownedItemsProfitLoss.percentage)
						end
					else
						v93 = v81:getSellPrice()
						v94 = g_i18n:formatMoney(v93)
					end
				else
					v94 = "-"
					v93 = 0
				end
				v82.columns[InGameMenuStatisticsFrame.COLUMN_VALUE] = {
					["text"] = v94,
					["value"] = v93
				}
				local v95 = p78.vehicles
				table.insert(v95, v82)
			end
		end
	end
	p78:updateView()
end
InGameMenuStatisticsFrame.updateVehicles = Utils.overwrittenFunction(InGameMenuStatisticsFrame.updateVehicles, extendedSelling.InGameMenuStatisticsFrame_updateVehicles)